/*typedef.h*/

#ifndef H_TYPEDEF
#define H_TYPEDEF

typedef unsigned char     *RDATA_PTR;      /* ptr to arbitrary data          */
typedef unsigned char     *URDATA_PTR;     /* unsigned ptr to arbitrary data */
typedef unsigned char      RDATA_BYTE;     /* for raw data                   */

#if !defined(O_BINARY) && !defined(MAC) 
#define O_BINARY 0
#endif

typedef RDATA_PTR HDATA_PTR;
typedef short   *HSHORT_PTR;

typedef enum
{
        NEARNEIGH,
        BILINEAR,
        CUBIC,
        FRACTAL
} ITYPE;

typedef enum
{
        BIT24,
        BIT8
} OTYPE;

#ifndef H_RASTER_D

#define BITS_PER_BYTE 8L

/***************************************************************
@    NUM_TYPE
****************************************************************
Types of numbers
*/
typedef enum
{
    ONEBIT,         /* binary data (0 or 1)*/
    UINT_4BIT,      /* unsigned 4 bit data (0 -> 15)*/
    SINT_8BIT,      /* signed 8 bit data (-128 -> 127) */
    UINT_8BIT,      /* unsigned 8 bit data (0 -> 255) */
    SINT_16BIT,     /* signed 16 bit (-32768 -> 32767) */
    UINT_16BIT,     /* unsigned 16 bit ( 0 -> 64K) */
    DTED_16BIT,     /* Univac signed magnitude */
    RGB_24BIT,      /* RGB (0 -> 255) interleaved by pixel */
    RGB_48BIT,      /* RGB (0 -> 65535) interleaved by pixel */
    SINT_32BIT,     /* signed 32 bit */
    UINT_32BIT,     /* unsigned 32 bit */
    IEEE_32BIT,     /* same as float */
    IEEE_64BIT      /* same as double */
}  NUM_TYPE;
/*
Description:
These are the number types that can be stored in RASTER's.
*/

/***************************************************************
@    DATA_TYPE
****************************************************************
Type of data contained in a RASTER object
*/
typedef enum
{
    COLOR_INDIRECT, /* color index numbers */
    COLOR_RGB,      /* rgb values */
    DATA            /* non-color data values */
}  DATA_TYPE;
/*
Description:
COLOR_INDIRECT rasters contain a matrix of color-map
indices.
COLOR_RGB rasters contain an RGB true-color image.
DATA rasters contain data which must be rendered before
being displayed.
*/

/***************************************************************
@    PROJECTION
****************************************************************
Map projections
*/
typedef enum
{
    PROJ_UNDEFINED,
    PROJ_LAMBERT1,
    PROJ_LAMBERT2,
    PROJ_LINEAR,
    PROJ_MERCATOR,
    PROJ_UPS,
    PROJ_UTM,
    PROJ_GRAPH,
    PROJ_POLAR,
    PROJ_ARC,
    PROJ_TS
} PROJECTION;
/*
 * Projections to be added in the future: PROJ_BONNE, PROJ_MOLLWEIDE,
 * PROJ_ORTHOGRAPHIC, PROJ_STEREOGRAPHIC, PROJ_TRANSVERSE_MERCATOR,
 * 
 * Description:
 */

typedef enum
{
    /*DD,*/DDTEMP,
    DMS,
    /*GEOREF,*/
    MGRS,
    MIN,
    SECS,
    UTM
} GEOGRAPHIC;

#endif

typedef enum configuration_type
{
  CPZIE,
  GEZ,
  GEIE,
  RASTERFILE,
  UNDEFINED
} CONFTYPE;

typedef struct image_params
{
  FILE_SPEC  *fs;
  double      left;           /* Leftmost extent of the dataset in DD   */
  double      right;          /* Rightmost extent of the dataset in DD  */
  double      bottom;         /* Bottommost extent of the dataset in DD */
  double      top;            /* Topmost extent of the dataset in DD    */
  double      clat;           /* Center latitude in DD                  */
  double      clon;           /* Center longitude in DD                 */
  double      x_res;          /* X resoultion of the data in h_units    */
  double      y_res;          /* Y Resolution of the data in h_units    */
  long        width;          /* Output Image X size in pixels          */
  long        height;         /* Output Image y size in pixels          */
  long        zone;           /* UTM zone                               */
  double      zoom;
  double      data_min;       /* The minimum value of the data          */
  double      data_max;       /* The maximum value of the data          */
  PROJECTION  projection;
} IPARAMS;

typedef struct image
{
  IPARAMS   ip;
  ITYPE     itype;                         /* Interpolation Method     */
  OTYPE     otype;                         /* Output Type (24 or 8 bit)*/
  long      config_type;                   /* Area designation method  */
  void     *data;                          /* Image Data Pointer       */
  char      output_filename[SZ_FNAME + 1]; /* Default output file name */
  char      output_filetype[6];            /* Default output file type */
} IMAGE;

#ifndef ROUND
#define ROUND(X) ((X)<0 ? -(long)(-(X)+0.5) : (long)((X)+0.5))
#endif
#ifndef MAX
#define  MAX(m,n) (((m) > (n)) ? (m) : (n))
#endif
#ifndef MIN
#define  MIN(m,n) (((m) < (n)) ? (m) : (n))
#endif

typedef struct import_control_data
{
  char       top[21];
  char       bot[21];
  char       left[21];
  char       right[21];
  char       center_ns[21];
  char       center_ew[21];
  char       utm_zone[4];
  char       zoom[6];
  char       image_sz_x[6];
  char       image_sz_y[6];
  char       zoom_x[4];
  char       zoom_y[4];
  char       matrix_x[6];
  char       matrix_y[6];
  CONFTYPE   method;
  GEOGRAPHIC system;
  short      cancel;
  ITYPE      itype;
  OTYPE      otype;
} IMPORT_CONTROL_DATA;
#endif
