
#include <LosServ/los.h>

extern "C" {
#include <projects.h>
#include <Los/los.h>
  RASTER elev_raster;
  RASTER* prod_raster;
  PARAM_CALC           param;             /* calculations location      */
  HANDLES              handle;            /* handles to working buffers */
  SETTINGS_TYPE settings;   /* user entered settings                    */
  DIRECTORY home_dir;
}

long prod_height() { return prod_raster->data_matrix.height; }
long prod_width() { return prod_raster->data_matrix.width; }
unsigned char* prod_ghandle() { return (unsigned char*)prod_raster->data_matrix.ghandle; }
float prod_ext_left() { return prod_raster->data_hdr.extent.left; }
float prod_ext_right() { return prod_raster->data_hdr.extent.right; }
float prod_ext_bottom() { return prod_raster->data_hdr.extent.bottom; }
float prod_ext_top() { return prod_raster->data_hdr.extent.top; }
float prod_xres() { return prod_raster->data_hdr.x_res; }
float prod_yres() { return prod_raster->data_hdr.y_res; }

/* Function to stuff the global LOS structures with a DEM */

UsgsDem* load_dem(char* filename) {
    unsigned long length_in_bytes;

    UsgsDem* dem = new UsgsDem();
    dem->read(filename);

    /* stuff the global raster datastruct with the DEM */
    elev_raster.magic = MAGIC_RASTER;
    strcpy(elev_raster.data_hdr.source, "USGS DEM");
    if (dem->reference_system() == UsgsDem::GEOGRAPHIC)
      elev_raster.data_hdr.h_units = DD;
    else if (dem->reference_system() == UsgsDem::UTM)
      elev_raster.data_hdr.h_units = UTM;

    if (dem->elevation_units() == UsgsDem::METERS)
      elev_raster.data_hdr.v_units = VER_METERS;
    else if (dem->elevation_units() == UsgsDem::FEET)
      elev_raster.data_hdr.v_units = VER_FEET;

    if (dem->reference_system() == UsgsDem::GEOGRAPHIC)
      elev_raster.data_hdr.x_res = (1.0 / 3600.0) * 3.0;
    else if (dem->reference_system() == UsgsDem::UTM)
      elev_raster.data_hdr.x_res = 30.0;

    if (dem->reference_system() == UsgsDem::GEOGRAPHIC)
      elev_raster.data_hdr.y_res = (1.0 / 3600.0) * 3.0;
    else if (dem->reference_system() == UsgsDem::UTM)
      elev_raster.data_hdr.y_res = 30.0;

    if (dem->reference_system() == UsgsDem::GEOGRAPHIC)
      elev_raster.data_hdr.projection = PROJ_ARC;
    else if (dem->reference_system() == UsgsDem::UTM)
      elev_raster.data_hdr.projection = PROJ_UTM;

    if (dem->reference_system() == UsgsDem::GEOGRAPHIC) {
      elev_raster.data_hdr.center.lon = dem->swe() + 0.5;
      elev_raster.data_hdr.center.lat = dem->swn() + 0.5;
      elev_raster.data_hdr.extent.left = dem->swe();
      elev_raster.data_hdr.extent.right = dem->see();
      elev_raster.data_hdr.extent.bottom = dem->swn();
      elev_raster.data_hdr.extent.top = dem->nwn();
    }
    else if (dem->reference_system() == UsgsDem::UTM) {
#if 0
      PJ* proj = dem->projection();
      UV data;
      data.u = dem->swe();
      data.v = dem->swn();
      data = pj_inv(data, proj);
      double rswe = data.u;
      double rswn = data.v;

      data.u = dem->nwe();
      data.v = dem->nwn();
      data = pj_inv(data, proj);
      double rnwe = data.u;
      double rnwn = data.v;

      data.u = dem->nee();
      data.v = dem->nen();
      data = pj_inv(data, proj);
      double rnee = data.u;
      double rnen = data.v;

      data.u = dem->see();
      data.v = dem->sen();
      data = pj_inv(data, proj);
      double rsee = data.u;
      double rsen = data.v;
#endif
      double rswe = dem->swe();
      double rswn = dem->swn();
      double rnwe = dem->nwe();
      double rnwn = dem->nwn();
      double rnee = dem->nee();
      double rnen = dem->nen();
      double rsee = dem->see();
      double rsen = dem->sen();
      elev_raster.data_hdr.extent.left = rswe < rnwe ? 
	rswe/* * RAD_TO_DEG */ : 
	rnwe/* * RAD_TO_DEG */;
      elev_raster.data_hdr.extent.right = rsee > rnee ? 
	rsee/* * RAD_TO_DEG */ :
	rnee/* * RAD_TO_DEG */;
      elev_raster.data_hdr.extent.bottom = rswn < rsen ?
	rswn/* * RAD_TO_DEG */ :
	rsen/* * RAD_TO_DEG */;
      elev_raster.data_hdr.extent.top = rnwn > rnen ?
	rnwn/* * RAD_TO_DEG */ :
	rnen/* * RAD_TO_DEG */;
      elev_raster.data_hdr.center.lon = elev_raster.data_hdr.extent.left +
	(elev_raster.data_hdr.extent.right - elev_raster.data_hdr.extent.left)
	/ 2.0;
      elev_raster.data_hdr.center.lat = elev_raster.data_hdr.extent.bottom +
	(elev_raster.data_hdr.extent.top - elev_raster.data_hdr.extent.bottom)
	/ 2.0;
    }
    switch(dem->horizontal_datum()) {
    case UsgsDem::NULL_HDATUM:
      if (dem->reference_system() == UsgsDem::GEOGRAPHIC)
	elev_raster.data_hdr.datum.hor_datum = HD_WGS72;
      else if (dem->reference_system() == UsgsDem::UTM)
	elev_raster.data_hdr.datum.hor_datum = HD_NAD27;
      break;
    case UsgsDem::NAD27:
      elev_raster.data_hdr.datum.hor_datum = HD_NAD27;
      break;
    case UsgsDem::WGS72:

      elev_raster.data_hdr.datum.hor_datum = HD_WGS72;
      break;
    case UsgsDem::WGS84:
      elev_raster.data_hdr.datum.hor_datum = HD_WGS84;
      break;
    case UsgsDem::NAD83:
      elev_raster.data_hdr.datum.hor_datum = HD_WGS84;
      break;
    }
    elev_raster.data_hdr.datum.ver_datum = VD_MSL;
    elev_raster.data_hdr.accuracy.rel_hor = -1.E0;
    elev_raster.data_hdr.accuracy.abs_hor = -1.E0; // 130 meters?
    elev_raster.data_hdr.accuracy.rel_ver = -1.E0;
    elev_raster.data_hdr.accuracy.abs_ver = -1.E0; // 30 meters?
    elev_raster.data_hdr.date.year;
    elev_raster.data_hdr.date.month;
    elev_raster.data_hdr.date.day;
    elev_raster.data_hdr.date.hour;
    elev_raster.data_hdr.date.minute;
    elev_raster.data_hdr.date.second;
    elev_raster.data_hdr.zoom = 1.E0;
    elev_raster.data_hdr.data_min = dem->elev_min();
    elev_raster.data_hdr.data_max = dem->elev_max();
    elev_raster.data_matrix.width = dem->cols();
    elev_raster.data_matrix.height = dem->rows();
    elev_raster.data_matrix.bits_per_pixel = 16;
    elev_raster.data_matrix.bytes_per_row; // twice?
    elev_raster.data_matrix.num_type = SINT_16BIT;
    elev_raster.data_matrix.data_type = DATA;

   /* set the palette and LUT pointers */
   elev_raster.palette = (PALETTE *)NULL;
   elev_raster.lut     = (LUT *)NULL;

   /* calculate the data matrix size */
   length_in_bytes = (elev_raster.data_matrix.width * elev_raster.data_matrix.height
                      * elev_raster.data_matrix.bits_per_pixel) / BITS_PER_BYTE;

   /* allocate the raster memory */
   elev_raster.data_matrix.ghandle = xvt_gmem_alloc(length_in_bytes);
   if (elev_raster.data_matrix.ghandle == (GHANDLE)NULL)
      return NULL /*STAT_MEM_ALLOC_ERR*/;

   /* read the data into ghandle pointer */
   for (int row = 0; row < elev_raster.data_matrix.height; row++)
     for (int col = 0; col < elev_raster.data_matrix.width; col++)
       ((short*)elev_raster.data_matrix.ghandle)
	 [row * elev_raster.data_matrix.width + col] =
	 dem->value(row, col);

   /* everything OK */
   return dem;
}

#if 0
DTED* los_load_dted(char* filename) {
    unsigned long length_in_bytes;

    DTED* dted = new DTED();
    dted->Read(filename, 1);

    /* stuff the global raster datastruct with the DEM */
    elev_raster.magic = MAGIC_RASTER;
    strcpy(elev_raster.data_hdr.source, "DTED");
    elev_raster.data_hdr.h_units = DD;
    elev_raster.data_hdr.v_units = VER_METERS;

    elev_raster.data_hdr.x_res = (1.0 / 3600.0) * dted->GetLonSecInterval();
    elev_raster.data_hdr.y_res = (1.0 / 3600.0) * dted->GetLatSecInterval();

    elev_raster.data_hdr.projection = PROJ_ARC;

    elev_raster.data_hdr.center.lon = dted->GetSWLon() + 0.5;
    elev_raster.data_hdr.center.lat = dted->GetSWLat() + 0.5;
    elev_raster.data_hdr.extent.left = dted->GetSWLon();
    elev_raster.data_hdr.extent.right = dted->GetNELon();
    elev_raster.data_hdr.extent.bottom = dted->GetSWLat();
    elev_raster.data_hdr.extent.top = dted->GetNELat();

    elev_raster.data_hdr.datum.hor_datum = HD_WGS84;

    elev_raster.data_hdr.datum.ver_datum = VD_MSL;
    elev_raster.data_hdr.accuracy.rel_hor = -1.E0;
    elev_raster.data_hdr.accuracy.abs_hor = -1.E0; // 130 meters?
    elev_raster.data_hdr.accuracy.rel_ver = -1.E0;
    elev_raster.data_hdr.accuracy.abs_ver = -1.E0; // 30 meters?
    elev_raster.data_hdr.date.year;
    elev_raster.data_hdr.date.month;
    elev_raster.data_hdr.date.day;
    elev_raster.data_hdr.date.hour;
    elev_raster.data_hdr.date.minute;
    elev_raster.data_hdr.date.second;
    elev_raster.data_hdr.zoom = 1.E0;
    elev_raster.data_hdr.data_min = dted->elev_min();
    elev_raster.data_hdr.data_max = dted->elev_max();
    elev_raster.data_matrix.width = dted->cols();
    elev_raster.data_matrix.height = dted->rows();
    elev_raster.data_matrix.bits_per_pixel = 16;
    elev_raster.data_matrix.bytes_per_row; // twice?
    elev_raster.data_matrix.num_type = SINT_16BIT;
    elev_raster.data_matrix.data_type = DATA;

   /* set the palette and LUT pointers */
   elev_raster.palette = (PALETTE *)NULL;
   elev_raster.lut     = (LUT *)NULL;

   /* calculate the data matrix size */
   length_in_bytes = (elev_raster.data_matrix.width * elev_raster.data_matrix.height
                      * elev_raster.data_matrix.bits_per_pixel) / BITS_PER_BYTE;

   /* allocate the raster memory */
   elev_raster.data_matrix.ghandle = xvt_gmem_alloc(length_in_bytes);
   if (elev_raster.data_matrix.ghandle == (GHANDLE)NULL)
      return NULL /*STAT_MEM_ALLOC_ERR*/;

   /* read the data into ghandle pointer */
   for (int row = 0; row < elev_raster.data_matrix.height; row++)
     for (int col = 0; col < elev_raster.data_matrix.width; col++)
       ((short*)elev_raster.data_matrix.ghandle)
	 [row * elev_raster.data_matrix.width + col] =
	 dted->value(row, col);

   /* everything OK */
   return dted;
}
#endif
