/*
 * Copyright 1994, 1995 Vectaport Inc., Cartoactive Systems
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#include <Map/mapfiles.h>

#include <OS/directory.h>
#include <OS/string.h>

#include <iostream.h>
#include <string.h>


/*****************************************************************************/

MapFiles::MapFiles (const char* name, const char* path) {
    if (name) {
	_name = new char[strlen(name)+1];
	strcpy( _name, name );
    } else
	_name = nil;

    if (path) {
	_path = new char[strlen(path)+1];
	strcpy( _path, path );
    } else
	_path = nil;

}

MapFiles::~MapFiles () {
    delete _name;
    delete _path;
}

const char* MapFiles::description() const { return "generic description"; }

char** MapFiles::dir_names (const char* path, int &ndirs) {
    Directory* dir = Directory::open(path);
    if (dir) {
    int nfiles = dir->count();
    ndirs = 0;
    for (int i=2; i<nfiles; i++)  // skip "." and ".."
	if (dir->is_directory(i)) ndirs++;
    if (!ndirs) {
	delete dir;
	return NULL;
    }
    char **dirnames = new char*[ndirs];
    int dirnum = 0;
    for (int i=2; i<nfiles; i++) 
	if (dir->is_directory(i)) 
	   dirnames[dirnum++] = strdup(dir->name(i)->string());
    delete dir;
    return dirnames;
    }
    else {
        ndirs = 0;
	return NULL;
    }
}

void MapFiles::set_path(const char* p) {
    delete _path;
    if (p) 
        _path = strdup(p);
    return;
}
