/*
 * Copyright (c) 1995 Cartoactive Systems
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#include <MapUnidraw/gscmd.h>
#include <MapUnidraw/gseditor.h>
#include <MapUnidraw/gsview.h>
#include <MapUnidraw/mapeditor.h>
#include <MapUnidraw/maptheme.h>
#include <MapUnidraw/mapviewstate.h>
#include <MapUnidraw/thchooser.h>
#include <Unidraw/Graphic/pspaint.h>
#include <Unidraw/catalog.h>
#include <Unidraw/editor.h>
#include <Unidraw/unidraw.h>
#include <InterViews/session.h>
#include <InterViews/style.h>
#include <InterViews/window.h>
#include <IV-look/field.h>
#include <OS/string.h>

/*****************************************************************************/

GSBrushCmd::GSBrushCmd(GSEditor* ed, PSBrush* br)
: BrushCmd((Editor*)nil, br)
{
    gseditor = ed;
}

void GSBrushCmd::Execute() {
    ObservableGS* gs = gseditor->GS();
    if (gs)
	gs->SetBrush(GetBrush());
}

/*****************************************************************************/

GSColorCmd::GSColorCmd(GSEditor* ed, PSColor* fg, PSColor* bg)
: ColorCmd((Editor*)nil, fg, bg)
{
    gseditor = ed;
}

void GSColorCmd::Execute() {
    ObservableGS* gs = gseditor->GS();
    if (gs) {
	PSColor* fg = (_fg == nil) ? gs->GetFgColor() : _fg;
        PSColor* bg = (_bg == nil) ? gs->GetBgColor() : _bg;
	gs->SetColors(fg, bg);
    }
}

/*****************************************************************************/

GSPatternCmd::GSPatternCmd(GSEditor* ed, PSPattern* pat)
: PatternCmd((Editor*)nil, pat)
{
    gseditor = ed;
}

void GSPatternCmd::Execute() {
    ObservableGS* gs = gseditor->GS();
    if (gs)
	gs->SetPattern(GetPattern());
}

/*****************************************************************************/

GSEditCmd::GSEditCmd(Editor* ed)
: Command(ed)
{
}

GSEditCmd::GSEditCmd(ControlInfo* i)
: Command(i)
{
}

void GSEditCmd::Execute() {
    TransientWindow* win = new TransientWindow(((MapEditor*)GetEditor())->gseditor());
    win->transient_for(GetEditor()->GetWindow());
    win->map();
}

/*****************************************************************************/

ThemeCmd::ThemeCmd(Editor* ed)
: Command(ed)
{
}

ThemeCmd::ThemeCmd(ControlInfo* i)
: Command(i)
{
}

void ThemeCmd::Execute() {
    TransientWindow* win = new TransientWindow(((MapEditor*)GetEditor())->thchooser());
    win->transient_for(GetEditor()->GetWindow());
    win->map();
}

/*****************************************************************************/

AddThemeCmd::AddThemeCmd(Editor* ed)
: Command(ed)
{
}

AddThemeCmd::AddThemeCmd(ControlInfo* i)
: Command(i)
{
}

void AddThemeCmd::Execute() {
    GSEditor* gseditor = ((MapEditor*)GetEditor())->gseditor();
    ThemeChooser* thchooser = ((MapEditor*)GetEditor())->thchooser();
    char* desc = (char*) gseditor->descfield()->text()->string();
    char* expr = (char*) gseditor->exprfield()->text()->string();
    ObservableGS* gs = gseditor->GS();
    MapTheme* theme = new MapTheme(desc, expr, gs);
    ((MapEditor*)GetEditor())->GetMapViewState()->append(theme);
    thchooser->append(theme);
}
