/*
 * Copyright 1994, 1995 Vectaport Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#include <Map/mapfeature.h>
#include <MapUnidraw/mapviewstate.h>
#include <Unidraw/globals.h>

#include <InterViews/transformer.h>

/*****************************************************************************/

MapViewState::MapViewState() {
    _trans = nil;
    _normtrans = true;
    _proj = nil;
    _implicit_proj = nil;
    _themes = new MapThemeList;
    _default_gs = new FullGraphic;
    _default_gs->FillBg(false);
    _default_gs->SetColors(psblack, pswhite);
    _default_gs->SetPattern(psnonepat);
    _default_gs->SetBrush(pssingle);
    _default_gs->SetFont(psstdfont);
}

MapViewState::MapViewState(PJ* p) {
    _trans = nil;
    _proj = nil;
    _implicit_proj = nil;
    _themes = new MapThemeList;
    defproj(p);
}

MapViewState::~MapViewState() {
    if (_trans)
	_trans->unref();
    if (_proj)
	pj_free(_proj);
}

Transformer* MapViewState::deftrans() const {
    return _trans;
}

PJ* MapViewState::defproj() const {
    return _proj;
}

void MapViewState::settrans(Transformer* t) {
    if (_trans)
	_trans->unref();
    _trans = t;
    if (_trans)
	_trans->ref();
}

void MapViewState::defproj(PJ* p) {
    if (_proj)
	pj_free(_proj);
    _proj = p;
}

void MapViewState::implicitproj(PJ* p) {
    if (_implicit_proj)
	pj_free(_implicit_proj);
    _implicit_proj = p;
}

PJ* MapViewState::implicitproj() const {
    return _implicit_proj;
}

Graphic* MapViewState::themegs(MapFeature* f) const {
    for (int i = 0; i < _themes->count(); i++) {
	MapTheme* theme = _themes->item(i);
	if (theme->eval(f)) {
	    return theme->gs();
	}
    }
    return _default_gs;
}

long MapViewState::count() const {
    return _themes->count();
}

MapTheme* MapViewState::item(long index) const {
    return _themes->item(index);
}

void MapViewState::prepend(MapTheme* mt) {
    _themes->prepend(mt);
}

void MapViewState::append(MapTheme* mt) {
    _themes->append(mt);
}

void MapViewState::insert(long index, MapTheme* mt) {
    _themes->insert(index, mt);
}

void MapViewState::remove(long index) {
    _themes->remove(index);
}

void MapViewState::remove_all() {
    _themes->remove_all();
}
